#include "combo.h"
#include <mega32.h>

#define begin {
#define end   }

//states for the Genesis controller debouncer
enum pushStates {noPush=1, maybePush, push, maybeNoPush}; 

/*		Genesis gamepad pins
		 __________________
		 \ o5 o4 o3 o2 o1 /
		  \ o9 o8 o7 o6 /
		   \___________/

        Pin #	Select = GND	Select = Vcc
        1(PC2)	Up				Up
        2(PC3)	Down			Down
        3(PC4)	GND				Left
        4(PC5)	GND				Right
        5		+5VDC    		+5VDC
        6(PC6)	Button A		Button B
        7(PC0)	Select			Select
        8		Ground			Ground
        9(PC7)	Start			Button C
        
        All buttons are active-low logic, so they return '0' when pressed and '1' when released
        
        Reading the controller:
        1) set Select to 0
        2) delay
        3) read PINC and save the top 2 bits (Button A, Start)
        4) set Select to 1
        5) delay
        6) read PINC and combine bits 2-7 (Up, Down, Left, Right, Button B, Button C)
           with the two bits from before
        	
        Bits will be associated with: (7 to 0)
        Start, Button A, Button C, Button B, Right, Left, Down, Up
*/
//==================================
// readController
//
// Used to decode button presses on the Genesis controller.
// Requires clock tick of approximately 30ms.
// We call it once per frame.

void readController(void)
begin
	//read the controller
	PORTC.0 = 0;//set select to 0
	delay_us(5);//delay
	controller = PINC & 0b11000000;//save top two bits
	PORTC.0 = 1;//set select to 1
	delay_us(5);//delay
	controller = controller | (PINC >> 2);//save the rest of the bits 

	//state machine for debouncing    
	switch(pushState)
	begin
		case (noPush):
		/*
			If a button has been pressed set state to maybePush
			and store the controller value.
		*/    
		       if (controller != 0xff)
		       begin
		       		pushState = maybePush;
		       		prevController = controller;
		       end
		       break;                                                                  
		case (maybePush):
		/*
			If current controller value is the same as the previous
			set state to push, set pushFlag high, and initialize
			holdCount to zero.
			Otherwise return to noPush state.
		*/
		       	if (prevController==controller)
			begin
				pushState = push;
				pushFlag = 1;
				holdCount = 0;
			end  
			else pushState = noPush; 
			break;
		case (push):
		/*
			If current controller value is the same as the previous
			keep state as push and increment holdCount (counts how long
			the button has been held down). If button has been held down
			for a certain amount of time (~0.7s initially, fewer seconds
			aftwerwards) set pushFlag high.
			Otherwise set state to maybeNoPush.
		*/
			if (prevController==controller)
			begin  
				pushState = push;
				if (holdCount++ > 40) 
				begin
					pushFlag=1;
					holdCount=holdCountReset;
				end
			end
			else pushState = maybeNoPush;
			break;
		case (maybeNoPush):
		/*
			If current controller value is the same as the previous
			set pushState back to push.
			Otherswise set state to noPush and set pushFlag low.
		*/
		       	if (prevController==controller)
		       		pushState = push;
			else
			begin
				pushState = noPush;
				pushFlag = 0;
			end
			break;
	end
end